import { customRef, getCurrentInstance, onMounted, nextTick, onUnmounted, reactive, ref } from 'vue-demi';
import { throttle, debounce } from 'lodash';
import screenfull from 'screenfull';

const useDebouncedRef = (value, delay = 200) => {
    let timeout;
    return customRef((track, trigger) => {
        return {
            get() {
                track();
                return value;
            },
            set(newValue) {
                clearTimeout(timeout);
                timeout = setTimeout(() => {
                    value = newValue;
                    trigger();
                }, delay);
            }
        };
    });
};

/**
 * Call onMounted() if it's inside a component lifecycle, if not, run just call the function
 * @param fn
 * @param sync if set to false, it will run in the nextTick() of Vue
 */
function tryOnMounted(fn, sync = true) {
    if (getCurrentInstance()) {
        onMounted(fn);
    }
    else if (sync) {
        fn();
    }
    else {
        nextTick(fn);
    }
}
/**
 * Call onUnmounted() if it's inside a component lifecycle, if not, do nothing
 * @param fn
 */
function tryOnUnmounted(fn) {
    if (getCurrentInstance())
        onUnmounted(fn);
}

function useReactiveState(getter) {
    const state = reactive(getter());
    const update = () => {
        Object.assign(state, getter());
    };
    return [state, update];
}

/**
 * Reactive Resize API, When window resize, do someting.
 * @param {function} fn
 * @param {boolean} useCapture
 * @param {boolean} isThrottle: true: throttle | false: debounce
 * @return {function} dispatchResize
 */
const useResize = (fn, useCapture = false, isThrottle = false) => {
    const dispatchResize = () => {
        const e = document.createEvent('Event');
        e.initEvent('resize', true, true);
        window.dispatchEvent(e);
    };
    const waitFn = isThrottle ? throttle(fn, 500) : debounce(fn, 500);
    tryOnMounted(() => {
        window.addEventListener('resize', waitFn, useCapture);
    });
    tryOnUnmounted(() => {
        window.removeEventListener('resize', waitFn, useCapture);
    });
    return {
        dispatchResize
    };
};

/**
 * Reactive Fullscreen API build on top of screenfull.js
 * @param {HTMLElement} target
 * @return {boolean} isFullscreen
 * @return {function} toggle
 */
const useFullscreen = (target) => {
    const targetRef = ref(target || document?.querySelector('html'));
    const isFullscreen = ref(false);
    const toggleFullscreen = () => {
        screenfull.isEnabled && screenfull.toggle(targetRef.value);
    };
    tryOnMounted(() => {
        if (screenfull.isEnabled) {
            screenfull.on('change', () => {
                isFullscreen.value = screenfull.isFullscreen;
            });
        }
    });
    return {
        isFullscreen,
        toggleFullscreen
    };
};

export { tryOnMounted, tryOnUnmounted, useDebouncedRef, useFullscreen, useReactiveState, useResize };
